/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:30:57.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.ObjectiveQuestionStructure;
import cn.com.qmth.examcloud.core.print.entity.SubjectiveQuestionStructure;
import cn.com.qmth.examcloud.core.print.repository.ObjectiveQuestionStructureRepository;
import cn.com.qmth.examcloud.core.print.repository.SubjectiveQuestionStructureRepository;
import cn.com.qmth.examcloud.core.print.service.CommonService;
import cn.com.qmth.examcloud.core.print.service.QuestionStructureService;
import cn.com.qmth.examcloud.core.print.service.bean.questionstructure.PaperQuestionStructureInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class QuestionStructureServiceImpl implements QuestionStructureService {
    private static final Logger log = LoggerFactory.getLogger(QuestionStructureServiceImpl.class);
    @Autowired
    private SubjectiveQuestionStructureRepository subjectiveQuestionStructureRepository;
    @Autowired
    private ObjectiveQuestionStructureRepository objectiveQuestionStructureRepository;
    @Autowired
    private CommonService commonService;

    @Override
    public void savePaperQuestionStructure(@NotNull Long examId, @NotNull String paperId) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        //从题库端获取试卷的试题结构信息
        PaperQuestionStructureInfo info = commonService.findStructureByPaperId(examId, paperId);

        //先清理旧数据
        objectiveQuestionStructureRepository.deleteByExamIdAndPaperId(examId, paperId);
        subjectiveQuestionStructureRepository.deleteByExamIdAndPaperId(examId, paperId);

        //保存客观题数据
        List<ObjectiveQuestionStructure> objectives = info.getObjectives();
        if (objectives != null && !objectives.isEmpty()) {
            objectiveQuestionStructureRepository.save(info.getObjectives());
        }

        //保存主观题数据
        List<SubjectiveQuestionStructure> subjectives = info.getSubjectives();
        if (subjectives != null && !subjectives.isEmpty()) {
            subjectiveQuestionStructureRepository.save(info.getSubjectives());
        }
    }

    @Override
    public List<ObjectiveQuestionStructure> getObjectiveQuestionStructureList(Long examId, String paperId) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("examId", examId)
                .eq("paperId", paperId);

        Specification<ObjectiveQuestionStructure> spec = SpecUtils.buildSearchers(ObjectiveQuestionStructure.class, searches.build());
        List<ObjectiveQuestionStructure> list = objectiveQuestionStructureRepository.findAll(spec);
        return list;
    }

    @Override
    public List<SubjectiveQuestionStructure> getSubjectiveQuestionStructureList(Long examId, String paperId) {
        Check.isNull(examId, "考试ID不能为空！");
        Check.isEmpty(paperId, "试卷ID不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("examId", examId)
                .eq("paperId", paperId);

        Specification<SubjectiveQuestionStructure> spec = SpecUtils.buildSearchers(SubjectiveQuestionStructure.class, searches.build());
        List<SubjectiveQuestionStructure> list = subjectiveQuestionStructureRepository.findAll(spec);
        return list;
    }

}